# -*- coding: utf-8 -*-
"""
    test_carrier

    Test carrier logic.

    :copyright: (C) 2014 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import sys
import os
DIR = os.path.abspath(os.path.normpath(os.path.join(
    __file__, '..', '..', '..', '..', '..', 'trytond'
)))
if os.path.isdir(DIR):
    sys.path.insert(0, os.path.dirname(DIR))
import unittest
from decimal import Decimal

import trytond.tests.test_tryton
from trytond.transaction import Transaction
from trytond.tests.test_tryton import POOL, DB_NAME, CONTEXT, USER


class TestCarrier(unittest.TestCase):
    '''
    Test carrier
    '''
    def setUp(self):
        """
        Set up data used in the tests.
        this method is called before each test function execution.
        """
        trytond.tests.test_tryton.install_module('carrier_zone')

    def create_carrier(self):
        '''
        Create the carrier
        '''
        Carrier = POOL.get('carrier')
        ProductTemplate = POOL.get('product.template')
        Product = POOL.get('product.product')
        UOM = POOL.get('product.uom')
        Currency = POOL.get('currency.currency')
        Party = POOL.get('party.party')

        party, = Party.create([{
            'name': 'Carrier',
        }])
        usd = Currency(
            name='US Dollar', symbol="$", code="USD"
        )
        usd.save()

        template, = ProductTemplate.create([{
            'name': 'Carrier',
            'default_uom': UOM.search([
                ('name', '=', 'Unit'),
            ])[0].id,
            'type': 'service',
            'list_price': Decimal(0),
            'cost_price': Decimal(0),
        }])
        product, = Product.create([{
            'template': template.id,
        }])

        carrier, = Carrier.create([{
            'zone_currency': usd,
            'carrier_product': product.id,
            'party': party.id,
        }])

        return carrier

    def test0010createzone(self):
        '''
        Create a zone and pricelist for it
        '''
        with Transaction().start(DB_NAME, USER, context=CONTEXT):
            Country = POOL.get('country.country')
            CarrierZonePriceList = POOL.get('carrier.zone_price_list')

            carrier = self.create_carrier()

            united_states, = Country.create([{
                'name': 'United States',
                'code': 'US',
            }])
            CarrierZonePriceList.create([{
                'carrier': carrier.id,
                'country': united_states.id,
                'price': Decimal('100'),
            }])

    def test0020priceonaddress(self):
        '''
        Ensure that the right zone is found
        '''
        with Transaction().start(DB_NAME, USER, context=CONTEXT):
            CarrierZonePriceList = POOL.get('carrier.zone_price_list')
            Country = POOL.get('country.country')
            Subdivision = POOL.get('country.subdivision')
            Address = POOL.get('party.address')

            carrier = self.create_carrier()

            united_states, india = Country.create([
                {
                    'name': 'United States',
                    'code': 'US',
                },
                {
                    'name': 'India',
                    'code': 'IN',
                },
            ])

            california, colorado = Subdivision.create([
                {
                    'country': united_states.id,
                    'name': 'Calfornia',
                    'code': 'CA',
                    'type': 'state'
                },
                {
                    'country': united_states.id,
                    'name': 'Colorado',
                    'code': 'CO',
                    'type': 'state'
                },
            ])

            uttar_pradesh, = Subdivision.create([
                {
                    'country': india.id,
                    'name': 'Uttar Pradesh',
                    'code': 'UP',
                    'type': 'state'
                },
            ])

            zone_in_all, = CarrierZonePriceList.create([
                {
                    'carrier': carrier.id,
                    'country': india.id,
                    'price': Decimal('100'),
                },

            ])
            zone_us_ca, = CarrierZonePriceList.create([
                {
                    'carrier': carrier.id,
                    'country': united_states.id,
                    'subdivision': california.id,
                    'price': Decimal('100'),
                }
            ])

            # Get a perfect match on state and country
            zone = carrier.find_zone_for_address(
                Address(
                    country=united_states.id,
                    subdivision=california.id,
                )
            )
            self.assertTrue(zone)
            self.assertEqual(zone.country.id, united_states.id)
            self.assertEqual(zone.subdivision.id, california.id)

            # no record with given subdivision or subdivision=None
            zone = carrier.find_zone_for_address(
                Address(
                    country=united_states.id,
                    subdivision=colorado.id,
                )
            )
            self.assertFalse(zone)

            # Get a match of zone for state that doesn't exist in list
            # but a record exist where subdivsion isn't set
            zone = carrier.find_zone_for_address(
                Address(
                    country=india.id,
                    subdivision=uttar_pradesh.id,
                )
            )
            self.assertTrue(zone)
            self.assertEqual(zone.country.id, india.id)
            self.assertFalse(zone.subdivision)


def suite():
    """
    Define suite
    """
    test_suite = trytond.tests.test_tryton.suite()
    test_suite.addTests(
        unittest.TestLoader().loadTestsFromTestCase(TestCarrier)
    )
    return test_suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite())



# -*- coding: utf-8 -*-
"""
    __init__

    :copyright: (c) 2014 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from trytond.pool import Pool

from .carrier import Carrier, CarrierZonePriceList
from .sale import Sale
from .stock import ShipmentOut


def register():
    Pool.register(
        Carrier,
        CarrierZonePriceList,
        ShipmentOut,
        Sale,
        module='carrier_zone', type_='model'
    )



# -*- coding: utf-8 -*-
"""
    :copyright: (c) 2014 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
from trytond.model import ModelSQL, ModelView, fields
from trytond.pool import PoolMeta, Pool
from trytond.pyson import Eval, Bool
from trytond.transaction import Transaction

__all__ = ['Carrier', 'CarrierZonePriceList']
__metaclass__ = PoolMeta


class Carrier:
    __name__ = 'carrier'

    zone_currency = fields.Many2One(
        'currency.currency', 'Currency',
        states={
            'invisible': Eval('carrier_cost_method') != 'zone',
            'required': Eval('carrier_cost_method') == 'zone',
            'readonly': Bool(Eval('zone_price_list', [])),
        },
        depends=['carrier_cost_method', 'zone_price_list'])
    zone_price_list = fields.One2Many(
        'carrier.zone_price_list', 'carrier',
        'Price List',
        states={
            'invisible': Eval('carrier_cost_method') != 'zone',
        },
        depends=['carrier_cost_method'])
    zone_currency_digits = fields.Function(
        fields.Integer(
            'Zone Currency Digits', on_change_with=['zone_currency']
        ), 'on_change_with_zone_currency_digits'
    )

    @classmethod
    def __setup__(cls):
        super(Carrier, cls).__setup__()
        selection = ('zone', 'Zone')
        if selection not in cls.carrier_cost_method.selection:
            cls.carrier_cost_method.selection.append(selection)

    def on_change_with_zone_currency_digits(self, name=None):
        if self.zone_currency:
            return self.zone_currency.digits
        return 2

    def get_sale_price(self):
        Address = Pool().get('party.address')
        ZonePriceList = Pool().get('carrier.zone_price_list')

        price, currency_id = super(Carrier, self).get_sale_price()
        if self.carrier_cost_method == 'zone':
            zone = None
            if 'address' in Transaction().context:
                zone = self.find_zone_for_address(
                    Address(Transaction().context['address'])
                )
            elif 'zone' in Transaction().context:
                zone, = ZonePriceList.search([
                    ('carrier', '=', self.id),
                    ('id', '=', Transaction().context['zone']),
                ])
            if zone is not None:
                return zone.price, self.zone_currency.id
        return price, currency_id

    def get_purchase_price(self):
        Address = Pool().get('party.address')
        ZonePriceList = Pool().get('carrier.zone_price_list')

        price, currency_id = super(Carrier, self).get_purchase_price()
        if self.carrier_cost_method == 'zone':
            zone = None
            if 'address' in Transaction().context:
                zone = self.find_zone_for_address(
                    Address(Transaction().context['address'])
                )
            elif 'zone' in Transaction().context:
                zone, = ZonePriceList.search([
                    ('carrier', '=', self.id),
                    ('id', '=', Transaction().context['zone']),
                ])
            if zone is not None:
                return zone.price, self.zone_currency.id
        return price, currency_id

    def find_zone_for_address(self, address):
        """
        A helper function that finds the most matching zone from the given
        address.

        :param address: Active Record of the address
        :return: Active Record of the zone_price_list
        """
        CarrierZone = Pool().get('carrier.zone_price_list')

        zones = CarrierZone.search([
            ('country', '=', address.country),
            ('subdivision', '=', address.subdivision),
        ], limit=1)

        if not zones:
            zones = CarrierZone.search([
                ('country', '=', address.country),
                ('subdivision', '=', None),
            ], limit=1)

        if zones:
            return zones[0]


class CarrierZonePriceList(ModelSQL, ModelView):
    'Carrier Zone price List'
    __name__ = 'carrier.zone_price_list'

    carrier = fields.Many2One('carrier', 'Carrier', required=True, select=True)
    country = fields.Many2One(
        'country.country', 'Country', required=True, select=True
    )
    subdivision = fields.Many2One(
        'country.subdivision', 'Subdivision', select=True,
        domain=[('country', '=', Eval('country'))],
        depends=['country']
    )
    price = fields.Numeric(
        'Price', digits=(16, Eval('_parent_carrier.weight_currency_digits', 2))
    )

    # TODO add a sequence and order by sequence



# -*- coding: utf-8 -*-
"""
    :copyright: (c) 2014 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""

from trytond.pool import PoolMeta

__all__ = ['Sale']
__metaclass__ = PoolMeta


class Sale:
    __name__ = "sale.sale"

    @classmethod
    def __setup__(cls):
        super(Sale, cls).__setup__()
        if 'shipment_address' not in cls.carrier.on_change:
            cls.carrier.on_change.append('shipment_address')
        if 'shipment_address' not in cls.lines.on_change:
            cls.lines.on_change.append('shipment_address')

    def _get_carrier_context(self):
        "Add zone to carrier context"
        context = super(Sale, self)._get_carrier_context()

        if not self.carrier:
            return context
        elif self.carrier.carrier_cost_method != 'zone':
            return context

        if self.shipment_address:
            context['address'] = self.shipment_address.id

        return context



# -*- coding: utf-8 -*-
"""
    setup

    :copyright: (c) 2014 by Openlabs Technologies & Consulting (P) Limited
    :license: BSD, see LICENSE for more details.
"""
import re
import os
import ConfigParser
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

config = ConfigParser.ConfigParser()
config.readfp(open('tryton.cfg'))
info = dict(config.items('tryton'))
for key in ('depends', 'extras_depend', 'xml'):
    if key in info:
        info[key] = info[key].strip().splitlines()
major_version, minor_version, _ = info.get('version', '0.0.1').split('.', 2)
major_version = int(major_version)
minor_version = int(minor_version)

requires = []
for dep in info.get('depends', []):
    if not re.match(r'(ir|res|webdav)(\W|$)', dep):
        requires.append(
            'trytond_%s >= %s.%s, < %s.%s' % (
                dep, major_version, minor_version,
                major_version, minor_version + 1
            )
        )

MODULE = 'carrier_zone'
PREFIX = 'openlabs'
MODULE2PREFIX = {}

for dep in info.get('depends', []):
    if not re.match(r'(ir|res|webdav)(\W|$)', dep):
        requires.append(
            '%s_%s >= %s.%s, < %s.%s' % (
                MODULE2PREFIX.get(dep, 'trytond'), dep,
                major_version, minor_version, major_version, minor_version + 1
            )
        )

setup(
    name='%s_%s' % (PREFIX, MODULE),
    version=info.get('version'),
    description='Zone based carrier cost for Tryton',
    author='Openlabs Technologies & Consulting (P) Limited',
    author_email='info@openlabs.co.in',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Tryton',
        'Topic :: Office/Business',
    ],
    packages=[
        'trytond.modules.%s' % MODULE,
        'trytond.modules.%s.tests' % MODULE,
    ],
    package_dir={
        'trytond.modules.%s' % MODULE: '.',
        'trytond.modules.%s.tests' % MODULE: 'tests',
    },
    package_data={
        'trytond.modules.%s' % MODULE: info.get('xml', []) +
        ['tryton.cfg', 'locale/*.po', 'tests/*.rst', '*.odt'] +
        ['i18n/*.pot', 'i18n/pt_BR/LC_MESSAGES/*', 'view/*.xml'],
    },
    install_requires=requires,
    zip_safe=False,
    entry_points="""
        [trytond.modules]
        %s = trytond.modules.%s
    """ % (MODULE, MODULE),
    test_suite='tests.suite',
    test_loader='trytond.test_loader:Loader',
)
